import React, { FC, ReactElement, useMemo } from 'react';
import { range } from 'ramda';
import { addDays, isSameDay, isSameWeek, setDate } from 'date-fns';

import { calcStartOf } from '../utils';
import { CalendarLayoutEnum, CalendarTypeEnum } from './Calendar';
import { CalendarCell, CalendarCellProps } from './CalendarCell';

export type DaysViewProps = {
	View?: FC<CalendarCellProps>;
	borderCollapse?: boolean;
	enableBorder?: boolean;
	enableHover?: boolean;
	layout?: CalendarLayoutEnum;
	onClick?: (date: Date | undefined) => void;
	selected?: Date;
	type?: CalendarTypeEnum;
	value?: Date;
	weekOfMonth: number;
	workDate: Date;
};

export function WeekDayView(props: DaysViewProps): ReactElement {
	const {
		layout,
		View,
		enableHover,
		onClick,
		workDate,
		weekOfMonth,
		borderCollapse,
		value,
		enableBorder,
		selected,
		type,
	} = props;

	const getStartOf = (): Date => {
		if (layout === CalendarLayoutEnum.day) return workDate;
		const startOf = layout === CalendarLayoutEnum.month ? setDate(workDate, 1) : workDate;
		return calcStartOf(startOf, weekOfMonth);
	};

	const renderDays = useMemo(() => {
		const startOf = getStartOf();
		const columns = layout === CalendarLayoutEnum.day ? 1 : 7;
		return range(0, columns).map((dayOfWeek) => {
			const date = addDays(startOf, dayOfWeek++);
			const props = {
				date,
				isToday: isSameDay(date, new Date()),
				workDate,
				hover: layout === CalendarLayoutEnum.day ? false : enableHover,
				onClick,
				layout,
				dayOfWeek,
				children: date.getDate(),
				weekOfMonth,
				enableBorder,
				enableHover,
				type,
				...(type === CalendarTypeEnum.weeks
					? {
							borderCollapse: true,
							isSelected: selected ? isSameWeek(date, selected, { weekStartsOn: 1 }) : false,
					  }
					: { borderCollapse, isSelected: selected ? isSameDay(date, selected) : false }),
			};
			return View ? <View key={date.getTime()} {...props} /> : <CalendarCell key={date.getTime()} {...props} />;
		});
	}, [workDate, View, layout, borderCollapse, weekOfMonth, value, enableHover, enableBorder, selected, type]);

	return <>{renderDays}</>;
}
