import React, { FC, Fragment, useEffect, useMemo, useState } from 'react';
import { range } from 'ramda';
import { add, endOfMonth, endOfWeek, getWeeksInMonth, startOfMonth, startOfWeek, sub } from 'date-fns';

import { SizeEnum } from '@procyon/types/enums';
import { isNilOrEmpty } from '@procyon/utils';

import { Text } from '../../Text';
import { CalendarCell, CalendarCellProps } from './CalendarCell';
import { CalendarDays, CalendarDaysProps } from './CalendarDays';
import { Controller, ControllerProps } from './Controller';
import { WeekDayView } from './WeekDayView';

export const getWeekRange = (date: Date) => [
	startOfWeek(date, { weekStartsOn: 1 }),
	endOfWeek(date, { weekStartsOn: 1 }),
];

export type CalendarProps = {
	Control?: FC<ControllerProps>;
	Days?: FC<CalendarDaysProps>;
	TimeColumn?: FC;
	View?: FC<CalendarCellProps>;
	enableHover?: boolean;
	layout?: CalendarLayoutEnum;
	longDayNames?: boolean;
	onClick?: (date: Date) => void;
	onRangeChange?: (dateFrom: Date, dateEnd: Date) => void;
	selected?: Date;
	title?: string;
	type?: CalendarTypeEnum;
	value?: Date;
	weekOfMonth?: number;
};

export enum CalendarLayoutEnum {
	day = 'day',
	month = 'month',
	week = 'week',
}

export enum CalendarTypeEnum {
	days = 'days',
	months = 'months',
	weeks = 'weeks',
}

export function Calendar({
	value,
	onClick,
	onRangeChange,
	layout = CalendarLayoutEnum.month,
	type = CalendarTypeEnum.days,
	Control = Controller,
	Days = CalendarDays,
	title,
	View,
	TimeColumn,
	enableHover,
	selected,
	longDayNames,
}: CalendarProps): ReturnType<FC> {
	const [workDate, setWorkDate] = useState<Date>(value ?? new Date());

	const handleChangeRange = () => {
		switch (layout) {
			case CalendarLayoutEnum.month:
				onRangeChange?.(startOfMonth(workDate), endOfMonth(workDate));
				return;
			case CalendarLayoutEnum.week:
				onRangeChange?.(getWeekRange(workDate)[0], getWeekRange(workDate)[1]);
				return;
			case CalendarLayoutEnum.day:
				onRangeChange?.(workDate, workDate);
		}
	};

	const handleClickPrev = () => {
		switch (layout) {
			case CalendarLayoutEnum.month:
				if (type === CalendarTypeEnum.months) {
					setWorkDate(sub(workDate, { years: 1 }));
					return;
				}
				setWorkDate(sub(workDate, { months: 1 }));
				return;
			case CalendarLayoutEnum.week:
				setWorkDate(sub(workDate, { weeks: 1 }));
				return;
			case CalendarLayoutEnum.day:
				setWorkDate(sub(workDate, { days: 1 }));
		}
	};

	const handleClickNext = () => {
		switch (layout) {
			case CalendarLayoutEnum.month:
				if (type === CalendarTypeEnum.months) {
					setWorkDate(add(workDate, { years: 1 }));
					return;
				}
				setWorkDate(add(workDate, { months: 1 }));
				return;
			case CalendarLayoutEnum.week:
				setWorkDate(add(workDate, { weeks: 1 }));
				return;
			case CalendarLayoutEnum.day:
				setWorkDate(add(workDate, { days: 1 }));
		}
	};

	const handleClickSetToday = () => {
		setWorkDate(new Date());
	};

	const handleClick = (date: Date) => {
		onClick?.(date);
	};

	useEffect(() => {
		if (value && value !== workDate) {
			setWorkDate(value);
		}
	}, [value]);

	useEffect(() => {
		handleChangeRange();
	}, [workDate]);

	const DaysLayout = useMemo(() => {
		const end = layout === CalendarLayoutEnum.month ? getWeeksInMonth(workDate, { weekStartsOn: 1 }) : 1;
		return (
			<>
				<Days layout={layout} workDate={workDate} longDayNames={longDayNames} />
				{range(0, end).map((i: number) => (
					<div key={i} className={'calendar-row table-row'}>
						<WeekDayView
							selected={selected}
							weekOfMonth={i}
							layout={layout}
							enableHover={enableHover}
							View={View}
							onClick={handleClick}
							workDate={new Date(workDate)}
							value={value}
							type={type}
						/>
					</div>
				))}
			</>
		);
	}, [layout, selected, enableHover, View, Days, workDate, value, type, longDayNames]);

	const MonthsLayout = useMemo(
		() =>
			range(0, 3).map((rI) => (
				<div key={rI} className="calendar-row table-row">
					{range(1, 5).map((cI) => {
						const month = cI + 4 * rI;
						const props = {
							date: new Date(workDate.getFullYear(), month - 1, 1),
							workDate,
							layout,
							enableHover: enableHover,
							isSelected: selected?.getMonth() === month - 1 && workDate.getFullYear() === selected?.getFullYear(),
							isToday: new Date().getMonth() === month - 1 && workDate.getFullYear() === new Date().getFullYear(),
							type: type,
							onClick: handleClick,
						};
						return View ? (
							<View key={cI} {...props} />
						) : (
							<CalendarCell key={cI} {...props}>
								{month}
							</CalendarCell>
						);
					})}
				</div>
			)),
		[enableHover, workDate, type, View, layout, onClick]
	);

	return (
		<>
			{!isNilOrEmpty(title) && <Text size={SizeEnum.md}>{title}</Text>}
			<Control
				layout={layout}
				onNextClick={handleClickNext}
				onPrevClick={handleClickPrev}
				onTodayClick={handleClickSetToday}
				type={type}
				workDate={workDate}
			/>
			<div className="flex">
				{TimeColumn ? <TimeColumn /> : <Fragment />}
				<div className="calendar table">
					<div className="table-row-group">
						{type === CalendarTypeEnum.days && DaysLayout}
						{type === CalendarTypeEnum.weeks && DaysLayout}
						{type === CalendarTypeEnum.months && MonthsLayout}
					</div>
				</div>
			</div>
		</>
	);
}
