import React, { useCallback, useEffect, useState } from 'react';
import clsx from 'clsx';

import { StatusEnum } from '@procyon/types/enums';
import { InputElementType } from '@procyon/types/form';

import { CheckSquareIcon } from './Icons/CheckSquare';
import { SquareIcon } from './Icons/Square';

export type CheckboxProps<V = boolean> = Omit<
	{ checked?: boolean; description?: string } & InputElementType<V>,
	'value'
>;

export const Checkbox: React.FC<CheckboxProps> = (props) => {
	const { label, description, name, status = StatusEnum.none, disabled } = props;

	const [checked, setChecked] = useState(false);

	useEffect(() => {
		setChecked(props.checked || false);
	}, [props.checked]);

	const handleClick = useCallback(
		(e: React.MouseEvent<HTMLElement>): void => {
			if (props.onChange) {
				props.onChange(!checked, e);
			}
			setChecked(!checked);
		},
		[checked]
	);

	const handleBlur = useCallback(
		(e: React.FocusEvent<HTMLInputElement>): void => {
			if (props.onBlur) {
				props.onBlur(checked, e);
			}
		},
		[checked]
	);

	return (
		<div
			className={clsx('field-container checkbox-container', {
				'field-container--info': status === StatusEnum.info,
				'field-container--success': status === StatusEnum.success,
				'field-container--warning': status === StatusEnum.warning,
				'field-container--error': status === StatusEnum.error,
				'field-container--nolabel': !label
			})}>
			<input
				id={name}
				name={name}
				className="hidden field-input"
				type="checkbox"
				onBlur={handleBlur}
				disabled={disabled}
			/>
			{label && (
				<label className="field-label" htmlFor={name} onClick={handleClick}>
					{label}
				</label>
			)}
			<div className="field-checkbox" onClick={handleClick}>
				{checked ? <CheckSquareIcon className="mr-1 field-icon" /> : <SquareIcon className="mr-1 field-icon" />}
				{description}
			</div>
		</div>
	);
};
