import React, { useCallback, useEffect, useState } from 'react';
import { omit } from 'ramda';
import { format as formatFn } from 'date-fns';

import { TodayIcon } from '@procyon/components/Icons/Today';
import { useModal } from '@procyon/components/Modal/useModal';
import { InputElementType } from '@procyon/types/form';

import { TextField } from '../TextField';
import { DatePickerModal } from './components/DatePickerModal';

export const DefaultDateFormat = 'dd.MM.yyyy';

export type DatePickerProps<V = Date> = Omit<
	{
		format?: string;
	} & InputElementType<V>,
	'onFocus' | 'onBlur'
>;

export const DatePicker: React.FC<DatePickerProps> = (props) => {
	const { onChange, label, name, format, ...others } = props;

	const [openModal, closeModal] = useModal(name, {
		title: label,
		style: { width: 400 },
	});

	const [value, setValue] = useState<Date | null>();

	useEffect(() => {
		if (props.value !== value) {
			setValue(props.value);
		}
	}, [props.value]);

	const handleDateClick = (value: Date): void => {
		setValue(value);
		onChange?.(value, null);
	};

	const modalComponent = useCallback(
		(props) => (
			<DatePickerModal
				format={format ?? DefaultDateFormat}
				name={name}
				value={value}
				closeModal={closeModal}
				onChange={handleDateClick}
				{...props}
			/>
		),
		[format, name, value],
	);

	const handleOpenModal = (): void => {
		openModal({
			Component: modalComponent,
		});
	};

	return (
		<TextField
			name={name}
			label={label}
			value={value ? formatFn(value, props.format ?? DefaultDateFormat) : undefined}
			{...omit(['value', 'onClick'], others)}
			leftIcon={<TodayIcon onClick={handleOpenModal} className="field-icon" />}
		/>
	);
};
