import React, { FC, useEffect, useRef, useState } from 'react';
import clsx from 'clsx';

import { t } from '@procyon/localization/message';

const calcOffset = (el: HTMLDivElement) => {
	const rect = el.getBoundingClientRect(),
		scrollLeft = window.scrollX || document.documentElement.scrollLeft,
		scrollTop = window.scrollY || document.documentElement.scrollTop;
	return {
		top: rect.top + scrollTop,
		left: rect.left + scrollLeft,
	};
};

export type HumanVerifyProps = {
	label?: string;
	onVerified: () => void;
};

export const HumanVerify: FC<HumanVerifyProps> = ({ onVerified, label }) => {
	const [isBot, setIsBot] = useState(true);

	const wrapperRef = useRef<HTMLDivElement>(null);
	const sliderRef = useRef<HTMLDivElement>(null);

	const handleMove = (e: any) => {
		if (!wrapperRef.current || !sliderRef.current) {
			return;
		}

		if (!isBot) {
			return;
		}

		const pageX = e.pageX || e.changedTouches[0].pageX;

		const wrapperOffset = calcOffset(wrapperRef.current);
		let left = pageX - wrapperOffset.left - sliderRef.current.offsetWidth / 2;
		const maxRight = wrapperRef.current.offsetWidth - sliderRef.current.offsetWidth - 2;

		if (left <= 0) {
			left = 0;
		} else if (left >= maxRight) {
			left = maxRight;
		}

		sliderRef.current.style.left = left + 'px';

		if (left === maxRight) {
			setIsBot(false);
			onVerified();
		}
	};

	useEffect(() => {
		const onMouseDownListener = () => {
			sliderRef.current?.addEventListener('mousemove', handleMove);
		};
		const onMouseUpListener = () => {
			sliderRef.current?.removeEventListener('mousemove', handleMove);
		};

		sliderRef.current?.addEventListener('mousedown', onMouseDownListener);
		sliderRef.current?.addEventListener('touchmove', handleMove);
		document.addEventListener('mouseup', onMouseUpListener);

		return () => {
			sliderRef.current?.removeEventListener('mousemove', handleMove);
			sliderRef.current?.removeEventListener('touchmove', handleMove);
			sliderRef.current?.removeEventListener('mousedown', onMouseDownListener);
			document.removeEventListener('mouseup', onMouseUpListener);
		};
	}, [sliderRef.current]);

	return (
		<div
			className={clsx('field-container humanverify-container', {
				'field-container--nolabel': !label,
			})}>
			{label && <label className="field-label">{label}</label>}
			<div
				ref={wrapperRef}
				className="hv-wrapper"
				style={
					{
						'--hv-slider-content-before': `"${t('procyon.common.yes')}"`,
						'--hv-slider-content-after': `"${t('procyon.common.no')}"`,
					} as any
				}>
				<div ref={sliderRef} className={clsx('hv-slider', { 'hv-slider--ok': !isBot })}></div>
			</div>
		</div>
	);
};
