import React, { useEffect, useMemo, useState } from 'react';
import { compose, filter, findIndex, includes, prop, propEq, toLower } from 'ramda';
import clsx from 'clsx';
import scrollIntoView from 'scroll-into-view-if-needed';

import { Option } from '@procyon/types/options';
import { isNilOrEmpty } from '@procyon/utils';

import { TextField } from './TextField';

type IProps = {
	autoComplete?: boolean;
	className?: string;
	compact?: boolean;
	enableAutoScroll?: boolean;
	name: string;
	noBorder?: boolean;
	onChange?: (options: Option) => void;
	options: Option[];
	scrollToCode?: Option['code'];
	selectedCode?: Option['code'];
};

const List: React.FC<IProps> = ({
	options,
	autoComplete,
	onChange,
	compact,
	name,
	scrollToCode,
	className,
	selectedCode,
	enableAutoScroll,
}) => {
	const [index, setIndex] = useState(0);
	const [selectedItemCode, setSelectedItemCode] = useState(selectedCode);
	const [searchResults, setSearchResults] = useState(options);

	const scrollToItem = (index: number) => {
		if (enableAutoScroll) {
			const element = document.querySelector(`#list-${name}_${index}`);
			scrollIntoView(element as any, {
				scrollMode: 'if-needed',
				behavior: 'smooth',
				block: 'center',
			});
		}
	};

	useEffect(() => {
		if (!isNilOrEmpty(selectedCode) && selectedCode !== selectedItemCode) {
			setSelectedItemCode(selectedCode);
		}
	}, [selectedCode, selectedItemCode]);

	const setCurrentOption = (index: number) => () => {
		setIndex(index);
	};

	const setActiveOption = (option: Option) => () => {
		setSelectedItemCode(option.code);
		onChange?.(option);
	};

	const handleKeyUp = (event: KeyboardEvent): void => {
		let newIndex = index;
		switch (event.key) {
			case 'ArrowDown':
				newIndex += 1;
				break;
			case 'ArrowUp':
				newIndex -= 1;
				break;
			case 'Enter':
				setActiveOption(searchResults[index])();
				return;
			default:
				return;
		}
		event.preventDefault();
		if (newIndex < 0) {
			newIndex = 0;
		} else if (newIndex > searchResults.length - 1) {
			newIndex = searchResults.length - 1;
		}
		setIndex(newIndex);
		scrollToItem(newIndex);
	};

	useEffect(() => {
		window.addEventListener('keydown', handleKeyUp);
		return () => {
			window.removeEventListener('keydown', handleKeyUp);
		};
	}, [handleKeyUp]);

	useEffect(() => {
		if (options && selectedItemCode) {
			const optionIndex = findIndex(propEq(selectedItemCode, 'code'), options);
			if (optionIndex !== -1) {
				setIndex(optionIndex);
				scrollToItem(optionIndex);
			}
		}
		if (options && scrollToCode && !selectedItemCode) {
			const optionIndex = findIndex(propEq(scrollToCode, 'code'), options);
			if (optionIndex !== -1) {
				scrollToItem(optionIndex);
			}
		}
	}, [options, selectedItemCode, name]);

	const handleSearch = (value: string) => {
		setSearchResults(filter<Option>(compose(includes(toLower(value)), toLower, prop('name')))(options));
		setIndex(0);
	};

	const items = useMemo(() => {
		if (!isNilOrEmpty(searchResults)) {
			return searchResults.map((option, i) => {
				return (
					<div
						key={option.code}
						id={`list-${name}_${i}`}
						className={clsx('list', {
							'list-compact': compact,
							'list--focus': index === i,
							'list--active': selectedItemCode === option.code,
						})}
						onClick={setActiveOption({ code: option.code, name: option.name })}
						onMouseEnter={setCurrentOption(i)}>
						<div>{option.name}</div>
						{option.description && <div>{option.description}</div>}
					</div>
				);
			});
		} else {
			return <div>Nic k zobrazení</div>;
		}
	}, [searchResults, index, selectedItemCode, name]);

	return (
		<div>
			{autoComplete && (
				<div className="pb-4">
					<TextField name="search" label="Hledat" onChange={handleSearch} autoFocus />
				</div>
			)}
			<div className={clsx('list-container', className)} onMouseLeave={setCurrentOption(-1)}>
				{items}
			</div>
		</div>
	);
};

List.displayName = 'List';

export default List;
