import React, { FC, MouseEvent, useEffect, useMemo, useState } from 'react';
import clsx from 'clsx';

import { CircleIcon } from '@procyon/components/Icons/Circle';
import { CircleFillIcon } from '@procyon/components/Icons/CircleFill';
import { StatusEnum } from '@procyon/types/enums';
import { InputElementType } from '@procyon/types/form';
import { Option } from '@procyon/types/options';

export type OptionViewProps = {
	icons?: { [key: Option['code']]: React.ReactNode };
};

export type InnerOptionsProps = {
	onClick: (code: Option['code'], e: MouseEvent<HTMLDivElement>) => void;
	option: Option;
	value: Option['code'] | null;
};

export type RadioButtonProps<V = Option['code']> = Omit<
	{
		View?: FC<OptionViewProps & InnerOptionsProps>;
		horizontal?: boolean;
		options: Option[];
		responsive?: boolean;
	} & InputElementType<V>,
	'onFocus' | 'onBlur' | 'onChange' | 'disabled'
>;

export const RadioButton: React.FC<RadioButtonProps> = (props) => {
	const {
		onClick,
		horizontal = false,
		label,
		options,
		View,
		status = StatusEnum.none,
		responsive,
		name,
		message
	} = props;

	const [value, setValue] = useState<Option['code'] | null>(null);

	useEffect(() => {
		if (props.value !== value) {
			setValue(props.value ?? null);
		}
	}, [props.value]);

	const handleClick = (code: Option['code'], e: MouseEvent<HTMLDivElement>): void => {
		onClick?.(code, e);
		setValue(code);
	};

	const elements = useMemo(() => {
		return options.map((option) => {
			const { code, name, disabled } = option;
			return (
				<div
					key={code}
					className={clsx({
						'mr-3': horizontal
					})}>
					{View ? (
						<View option={option} onClick={handleClick} value={value} />
					) : (
						<div
							className={clsx('flex field-radio', { 'field-radio--disabled': disabled })}
							onClick={(e) => (!disabled ? handleClick(code, e) : null)}>
							{value === code && !disabled ? (
								<CircleFillIcon className="field-icon mr-1" />
							) : (
								<CircleIcon className="field-icon mr-1" />
							)}
							{name}
						</div>
					)}
				</div>
			);
		});
	}, [value, options, name]);

	return (
		<>
			<div
				className={clsx('field-container radiobutton-container', {
					'field-container--info': status === StatusEnum.info,
					'field-container--success': status === StatusEnum.success,
					'field-container--warning': status === StatusEnum.warning,
					'field-container--error': status === StatusEnum.error
				})}>
				{label && <div className="field-label cursor-default mr-2 self-start">{label}</div>}
				<div
					className={clsx(
						'flex',
						responsive
							? 'flex-col sm:flex-row'
							: {
									'flex-col': !horizontal,
									'flex-row': horizontal
							  }
					)}>
					{elements}
				</div>
			</div>
			{message && <div>{message}</div>}
		</>
	);
};
