import React, { createContext, FC, ReactNode, useEffect, useMemo, useState } from 'react';
import { omit } from 'ramda';

import { StatusEnum } from '@procyon/types/enums';

export type Toaster = {
	id: string;
	message: string;
	status: StatusEnum;
	timeout?: number;
	title?: string;
};

export type ToasterId = Toaster['id'];

type ToasterContextType = {
	open: (toaster: Toaster) => void;
	toasters: Record<ToasterId, Toaster>;
};

const DefaultTimeout = 6000;
const timeouts: { [key: ToasterId]: any } = [];

export const ToasterContext = createContext<ToasterContextType>({
	toasters: {},
	open: () => {},
});

export const ToasterWrapper: FC<{ children: ReactNode }> = ({ children }) => {
	const [toasters, setToasters] = useState<Record<ToasterId, Toaster>>({});

	const autoHide = (toaster: Toaster) => {
		const { id } = toaster;
		setToasters({ ...toasters, [id]: toaster });
		timeouts[id] = setTimeout(() => {
			setToasters(omit([id], toasters));
		}, toaster.timeout ?? DefaultTimeout);
	};

	useEffect(
		() => () => {
			Object.keys(timeouts).forEach((timeoutId) => {
				clearTimeout(timeouts[timeoutId]);
			});
		},
		[]
	);

	useEffect(() => {
		window.addEventListener('showToaster', (e: CustomEventInit<Toaster>) => {
			if (e.detail) autoHide(e.detail);
		});
	}, []);

	const value = useMemo(
		() => ({
			toasters,
			open: autoHide,
		}),
		[toasters]
	);

	return <ToasterContext.Provider value={value}>{children}</ToasterContext.Provider>;
};

export const showToaster = (toaster: Toaster) => {
	window.dispatchEvent(new CustomEvent<Toaster>('showToaster', { detail: toaster }));
};
